@@ -62,16 +62,20 @@ int get_internal_node(int sequence_index)
 void get_sequence_for_sample_name(char * sequence_bases, char * sample_name)
 {
     int sequence_index;
-    sequence_index = find_sequence_index_from_sample_name( sample_name);
+    sequence_index = find_sequence_index_from_sample_name(sample_name);
     if(sequence_index < 0)
     {
-        printf("Couldnt find sequence name %s with index %d\n", sample_name,sequence_index);
+        printf("Could not find sequence name %s with index %d\n", sample_name,sequence_index);
         exit(1);
     }
-
-    memcpy(sequence_bases, sequences[sequence_index], size_of_string(sequences[sequence_index]) +1);
+    get_sequence_for_sample_index(sequence_bases, sequence_index);
+    
 }
 
+void get_sequence_for_sample_index(char * sequence_bases, int sequence_index)
+{
+    memcpy(sequence_bases, sequences[sequence_index], size_of_string(sequences[sequence_index]) +1);
+}
 
 void fill_in_unambiguous_gaps_in_parent_from_children(int parent_sequence_index, int * child_sequence_indices, int num_children)
 {
@@ -237,10 +241,8 @@ void filter_sequence_bases_and_rotate(char * reference_bases, char ** filtered_b
 }
 
 
-void set_number_of_recombinations_for_sample(char * sample_name, int number_of_recombinations)
+void set_number_of_recombinations_for_sample(int sample_index, int number_of_recombinations)
 {
-    int sample_index ;
-    sample_index = find_sequence_index_from_sample_name( sample_name);
     if( sample_index == -1)
     {
         return;
@@ -249,15 +251,12 @@ void set_number_of_recombinations_for_sample(char * sample_name, int number_of_r
 }
 
 
-void set_number_of_snps_for_sample(char * sample_name, int number_of_snps)
+void set_number_of_snps_for_sample(int sample_index, int number_of_snps)
 {
-    int sample_index ;
-    sample_index = find_sequence_index_from_sample_name( sample_name);
     if( sample_index == -1)
     {
         return;
     }
-
     ((sample_statistics *) statistics_for_samples[sample_index])->number_of_snps = number_of_snps;
 }
 
@@ -275,10 +274,8 @@ void set_number_of_blocks_for_sample(char * sample_name,int num_blocks)
 
 
 
-void set_genome_length_without_gaps_for_sample(char * sample_name, int genome_length_without_gaps)
+void set_genome_length_without_gaps_for_sample(int sample_index, int genome_length_without_gaps)
 {
-    int sample_index ;
-    sample_index = find_sequence_index_from_sample_name( sample_name);
     if( sample_index == -1)
     {
         return;
@@ -287,11 +284,10 @@ void set_genome_length_without_gaps_for_sample(char * sample_name, int genome_le
     ((sample_statistics *) statistics_for_samples[sample_index])->genome_length_without_gaps = genome_length_without_gaps;
 }
 
-void set_genome_length_excluding_blocks_and_gaps_for_sample(char * sample_name, int genome_length_excluding_blocks_and_gaps)
+void set_genome_length_excluding_blocks_and_gaps_for_sample(int sample_index, int genome_length_excluding_blocks_and_gaps)
 {
-	int sample_index ;
-	sample_index = find_sequence_index_from_sample_name( sample_name);
-    if( sample_index == -1)
+
+  if( sample_index == -1)
     {
 		return;
 	}
@@ -299,26 +295,34 @@ void set_genome_length_excluding_blocks_and_gaps_for_sample(char * sample_name,
 	((sample_statistics *) statistics_for_samples[sample_index])->genome_length_excluding_blocks_and_gaps = genome_length_excluding_blocks_and_gaps;
 }
 
-void set_number_of_branch_bases_in_recombinations(char * sample_name, int bases_in_recombinations)
+void set_number_of_branch_bases_in_recombinations(int sample_index, int bases_in_recombinations, int include_gaps)
 {
-    int sample_index ;
-    sample_index = find_sequence_index_from_sample_name( sample_name);
+
     if( sample_index == -1)
     {
         return;
     }
-    ((sample_statistics *) statistics_for_samples[sample_index])->branch_bases_in_recombinations = bases_in_recombinations;
+    
+    if (include_gaps > 0) {
+      ((sample_statistics *) statistics_for_samples[sample_index])->branch_bases_in_recombinations_including_gaps = bases_in_recombinations;
+    } else {
+      ((sample_statistics *) statistics_for_samples[sample_index])->branch_bases_in_recombinations = bases_in_recombinations;
+    }
 }
 
-void set_number_of_bases_in_recombinations(char * sample_name, int bases_in_recombinations)
+void set_number_of_bases_in_recombinations(int sample_index, int bases_in_recombinations, int include_gaps)
 {
-    int sample_index ;
-    sample_index = find_sequence_index_from_sample_name( sample_name);
-    if( sample_index == -1)
-    {
-        return;
-    }
+  if( sample_index == -1)
+  {
+      return;
+  }
+  
+  if (include_gaps > 0) {
+    ((sample_statistics *) statistics_for_samples[sample_index])->bases_in_recombinations_including_gaps = bases_in_recombinations;
+  } else {
     ((sample_statistics *) statistics_for_samples[sample_index])->bases_in_recombinations = bases_in_recombinations;
+  }
+  
 }
 
 
@@ -387,7 +391,6 @@ void load_sequences_from_multifasta_file(char filename[])
     get_sample_names_for_header(filename, phylip_sample_names, num_samples);
 
     int l;
-    i = 0;
     int sequence_number = 0;
 
     gzFile fp;
